## $Id: batching.py 12073 2014-11-27 21:11:32Z henrik $
##
## Copyright (C) 2014 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""Batch processing components for product objects.

Batch processors eat CSV files to add, update or remove large numbers
of certain kinds of objects at once.
"""
import grok
import unicodecsv as csv  # XXX: csv ops should move to dedicated module.
from time import time
from datetime import datetime
from zope.i18n import translate
from zope.interface import Interface
from zope.schema import getFields
from zope.component import queryUtility, getUtility, createObject
from zope.event import notify
from zope.catalog.interfaces import ICatalog
from hurry.workflow.interfaces import IWorkflowState, IWorkflowInfo
from waeup.ikoba.interfaces import (
    IBatchProcessor, FatalCSVError, IObjectConverter, IUserAccount,
    IObjectHistory, IGNORE_MARKER)
from waeup.ikoba.interfaces import IIkobaUtils
from waeup.ikoba.interfaces import MessageFactory as _
from waeup.ikoba.products.interfaces import IProduct
from waeup.ikoba.utils.batching import BatchProcessor


class ProductProcessor(BatchProcessor):
    """A batch processor for IProduct objects.
    """
    grok.implements(IBatchProcessor)
    grok.provides(IBatchProcessor)
    grok.context(Interface)
    util_name = 'productprocessor'
    grok.name(util_name)

    name = _('Product Processor')
    iface = IProduct

    location_fields = ['product_id',]
    factory_name = 'waeup.Product'

    mode = None

    def parentsExist(self, row, site):
        return 'products' in site.keys()

    def entryExists(self, row, site):
        return row['product_id'] in site['products'].keys()

    def getParent(self, row, site):
        return site['products']

    def getEntry(self, row, site):
        if not self.entryExists(row, site):
            return None
        parent = self.getParent(row, site)
        return parent.get(row['product_id'])

    def addEntry(self, obj, row, site):
        parent = self.getParent(row, site)
        parent.addProduct(obj)
        return

    def delEntry(self, row, site):
        product = self.getEntry(row, site)
        parent = self.getParent(row, site)
        if product is not None:
            grok.getSite().logger.info(
                '%s - Product removed' % product.product_id)
            del parent[product.product_id]
        return

    def updateEntry(self, obj, row, site, filename):
        """Update obj to the values given in row.
        """
        items_changed = super(ProductProcessor, self).updateEntry(
            obj, row, site, filename)
        # Log actions...
        location_field = self.location_fields[0]
        grok.getSite().logger.info(
            '%s - %s - %s - updated: %s'
            % (self.name, filename, row[location_field], items_changed))
        return
